# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SubWorkflowDefinition(UniversalBaseModel):
    workflow_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the workflow to execute as a sub-workflow
    """

    input: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Input data to pass to the sub-workflow
    """

    runtime_placeholders: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Maps placeholder tokens to their input data paths for sub-workflow
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
