# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Lang2FhirSearchDefinition(UniversalBaseModel):
    original_query: typing.Optional[str] = pydantic.Field(default=None)
    """
    Original natural language query
    """

    resource_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    FHIR resource type to search
    """

    search_parameters: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Prepared FHIR search parameters
    """

    required_fields: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Fields needed from search results
    """

    runtime_placeholders: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Maps placeholder tokens to their input data paths
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
